@@ -13,7 +13,7 @@
 #include <sstream>
 #include <iterator>
 
-// #include <pcrecpp.h>
+#include <pcrecpp.h>
 
 #include "procman.h"
 #include "lsof.h"
@@ -57,7 +57,7 @@ namespace
   };
 
 
-#ifdef FOOBAR
+#if 0
 
   class Lsof
     : public LsofBase
@@ -87,7 +87,7 @@ namespace
     { }
   };
 
-#elif 0
+#elif 1
 
   class Lsof
     : public LsofBase
@@ -250,8 +250,8 @@ namespace
       // lsof = new SimpleLsof(this->pattern(), this->case_insensitive);
       unsigned count = 0;
 
-      for (GList *i = this->procdata->info; i; i = i->next) {
-	ProcInfo &info(*static_cast<ProcInfo*>(i->data));
+      for (ProcInfo::Iterator it(ProcInfo::begin()); it != ProcInfo::end(); ++it) {
+	const ProcInfo &info(*it->second);
 
 	MatchSet matches;
 	lsof.search(info, std::inserter(matches, matches.begin()));
