@@ -25,7 +25,6 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
-#include <gnome.h>
 #include <bacon-message-connection.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <gconf/gconf-client.h>
@@ -38,9 +37,16 @@
 #include "interface.h"
 #include "proctable.h"
 #include "prettytable.h"
-#include "favorites.h"
 #include "callbacks.h"
 #include "smooth_refresh.h"
+#include "util.h"
+#include "gconf-keys.h"
+
+ProcData* ProcData::get_instance()
+{
+  static ProcData instance;
+  return &instance;
+}
 
 
 static void
@@ -55,6 +61,18 @@ tree_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data
 	
 }
 
+static void
+solaris_mode_changed_cb(GConfClient *client, guint id, GConfEntry *entry, gpointer data)
+{
+	ProcData *procdata = static_cast<ProcData*>(data);
+	GConfValue *value = gconf_entry_get_value (entry);
+
+	procdata->config.solaris_mode = gconf_value_get_bool(value);
+	proctable_update_all (procdata);
+}
+
+
+
 static void
 view_as_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 {
@@ -78,9 +96,6 @@ warning_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer d
 	if (g_str_equal (key, "/apps/procman/kill_dialog")) {
 		procdata->config.show_kill_warning = gconf_value_get_bool (value);
 	}
-	else {
-		procdata->config.show_hide_message = gconf_value_get_bool (value);
-	}
 }
 
 static void
@@ -95,7 +110,7 @@ timeouts_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer
 		procdata->config.update_interval = 
 			MAX (procdata->config.update_interval, 1000);
 
-		smooth_refresh_reset(procdata->smooth_refresh);
+		procdata->smooth_refresh->reset();
 
 		if(procdata->timeout) {
 			g_source_remove (procdata->timeout);
@@ -217,14 +232,11 @@ procman_data_new (GConfClient *client)
 	gint i;
 	glibtop_cpu cpu;
 
-	pd = g_new0 (ProcData, 1);
+	pd = ProcData::get_instance();
 	
 	pd->tree = NULL;
-	pd->info = NULL;
-	pd->pids = g_hash_table_new(NULL, NULL);
 	pd->selected_process = NULL;
 	pd->timeout = 0;
-	pd->blacklist = NULL;
 	pd->cpu_graph = NULL;
 	pd->mem_graph = NULL;
 	pd->net_graph = NULL;
@@ -241,14 +253,15 @@ procman_data_new (GConfClient *client)
 	pd->config.show_tree = gconf_client_get_bool (client, "/apps/procman/show_tree", NULL);
 	gconf_client_notify_add (client, "/apps/procman/show_tree", tree_changed_cb,
 				 pd, NULL, NULL);
+
+	pd->config.solaris_mode = gconf_client_get_bool(client, procman::gconf::solaris_mode.c_str(), NULL);
+	gconf_client_notify_add(client, procman::gconf::solaris_mode.c_str(), solaris_mode_changed_cb, pd, NULL, NULL);
+
+
 	pd->config.show_kill_warning = gconf_client_get_bool (client, "/apps/procman/kill_dialog", 
 							      NULL);
 	gconf_client_notify_add (client, "/apps/procman/kill_dialog", warning_changed_cb,
 				 pd, NULL, NULL);
-	pd->config.show_hide_message = gconf_client_get_bool (client, "/apps/procman/hide_message",
-							      NULL);
-	gconf_client_notify_add (client, "/apps/procman/hide_message", warning_changed_cb,
-				 pd, NULL, NULL);
 	pd->config.update_interval = gconf_client_get_int (client, "/apps/procman/update_interval", 
 							   NULL);
 	gconf_client_notify_add (client, "/apps/procman/update_interval", timeouts_changed_cb,
@@ -349,8 +362,6 @@ procman_data_new (GConfClient *client)
 	gdk_color_parse(color, &pd->config.net_out_color);
 	g_free (color);
 	
-	get_blacklist (pd, client);
-	
 	/* Sanity checks */
 	swidth = gdk_screen_width ();
 	sheight = gdk_screen_height ();
@@ -376,7 +387,9 @@ procman_data_new (GConfClient *client)
     	if (pd->config.num_cpus == 0)
     		pd->config.num_cpus = 1;
 
-	pd->smooth_refresh = smooth_refresh_new(&pd->config.update_interval);
+	// delayed initialization as SmoothRefresh() needs ProcData
+	// i.e. we can't call ProcData::get_instance
+	pd->smooth_refresh = new SmoothRefresh();
 
 	return pd;
 
@@ -388,11 +401,7 @@ procman_free_data (ProcData *procdata)
 
 	proctable_free_table (procdata);
 	g_string_chunk_free(procdata->users);
-	g_hash_table_destroy(procdata->pids);
-	pretty_table_free (procdata->pretty_table);
-	smooth_refresh_destroy(procdata->smooth_refresh);
-	g_free (procdata);
-	
+	delete procdata->smooth_refresh;
 }
 
 
@@ -572,8 +581,6 @@ procman_save_config (ProcData *data)
 	gconf_client_set_int (client, "/apps/procman/width", data->config.width, NULL);
 	gconf_client_set_int (client, "/apps/procman/height", data->config.height, NULL);	
 	gconf_client_set_int (client, "/apps/procman/current_tab", data->config.current_tab, NULL);
-	
-	save_blacklist (data, client);
 
 	gconf_client_suggest_sync (client, NULL);
 	
@@ -657,7 +664,6 @@ int
 main (int argc, char *argv[])
 {
 	guint32 startup_timestamp;
-	GnomeProgram *procman;
 	GConfClient *client;
 	ProcData *procdata;
 	BaconMessageConnection *conn;
@@ -668,9 +674,8 @@ main (int argc, char *argv[])
 
 	startup_timestamp = get_startup_timestamp();
 
-	procman = gnome_program_init ("gnome-system-monitor", VERSION, 
-				       LIBGNOMEUI_MODULE, argc, argv,
-				      GNOME_PARAM_APP_DATADIR,DATADIR, NULL);
+	gtk_init(&argc, &argv);
+	procman_debug("post gtk_init");
 
 	conn = bacon_message_connection_new ("gnome-system-monitor");
 	if (!conn) g_error("Couldn't connect to gnome-system-monitor");
@@ -705,12 +710,15 @@ main (int argc, char *argv[])
 
 	gnome_vfs_init ();
 	glibtop_init ();
+
+	procman_debug("end init");
 	
 	procdata = procman_data_new (client);
 	procdata->client = client;
-	pretty_table_new (procdata);
 
+	procman_debug("begin create_main_window");
 	create_main_window (procdata);
+	procman_debug("end create_main_window");
 	
 	proctable_update_all (procdata);
 
@@ -719,7 +727,8 @@ main (int argc, char *argv[])
 	g_assert(procdata->app);
 			
  	gtk_widget_show(procdata->app);
- 	
+       
+	procman_debug("begin gtk_main");
 	gtk_main ();
 	
 	procman_free_data (procdata);
